/*
 * Demoiselle Framework
 * Copyright (c) 2010 Serpro and other contributors as indicated
 * by the @author tag. See the copyright.txt in the distribution for a
 * full listing of contributors.
 *
 * Demoiselle Framework is an open source Java EE library designed to accelerate
 * the development of transactional database Web applications.
 *
 * Demoiselle Framework is released under the terms of the LGPL license 3
 * http://www.gnu.org/licenses/lgpl.html  LGPL License 3
 *
 * This file is part of Demoiselle Framework.
 *
 * Demoiselle Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 3 as published by
 * the Free Software Foundation.
 *
 * Demoiselle Framework is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Demoiselle Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
package br.gov.component.demoiselle.crud.layer.integration;

import java.lang.annotation.Annotation;

import javax.persistence.EntityManager;

import org.apache.log4j.Logger;

import br.gov.component.demoiselle.crud.layer.ICrudDAO;
import br.gov.component.demoiselle.crud.supercrud.ImplicitSuperCrudDAO;
import br.gov.framework.demoiselle.core.bean.IPojo;
import br.gov.framework.demoiselle.core.layer.integration.IFactory;
import br.gov.framework.demoiselle.core.layer.integration.Injection;
import br.gov.framework.demoiselle.core.layer.integration.InjectionContext;
import br.gov.framework.demoiselle.persistence.layer.integration.PersistenceEntityManagerFactory;
import br.gov.framework.demoiselle.util.Constant;
import br.gov.framework.demoiselle.util.config.ConfigurationLoader;
import br.gov.framework.demoiselle.web.layer.integration.WebLayerIntegrationException;

/**
 * Especial factory to inject an ICrudDAO.
 * It can be an user implementation or an internal implementation.
 *
 * @version %I%, %G%
 * @see br.gov.component.demoiselle.crud.layer.ICrudDAO
 * @see br.gov.component.demoiselle.crud.supercrud.ImplicitSuperCrudDAO
 */
public class CrudDAOFactory extends CrudFactory implements IFactory<ICrudDAO<?>> {

	private static Logger logger = Logger.getLogger(CrudDAOFactory.class);

	/**
	 *
	 * Instanciate the corret implementation of ICrudDAO.
	 *
	 * This method use InjectionContext from the AspectJ;
	 *
	 * @param ctx @see br.gov.framework.demoiselle.core.layer.integration.InjectionContext
	 * @return implementation of ICrudDAO
	 */
	public ICrudDAO<?> create(InjectionContext ctx) {
		Class<IPojo> pojo = getPojoClass(ctx.getClassType());
		return createDAO(pojo);

	}

	/**
	 *
	 * Instanciate the corret implementation of ICrudDAO.
	 *
	 * This method use de pojo parameter to match with the
	 * user patterns of packages, defined on configurator file.
	 *
	 * @param pojo
	 * @return implementation of ICrudDAO
	 */
	public ICrudDAO<? extends IPojo> createDAO(Class<? extends IPojo> pojo) {
		CrudFactoryConfig config = ConfigurationLoader.load(CrudFactoryConfig.class);
		if(config.hasDAO()){
			logger.debug("Using user pattern from "+ Constant.FRAMEWORK_CONFIGURATOR_FILE);
            String className = conventionForClassName(pojo.getName(),config.getRegexDAO(), config.getReplaceDAO());
			try {
	            return (ICrudDAO<?>) createWithLazyCreateProxy(Class.forName(className));
			} catch (ClassNotFoundException e) {
				logger.debug("No user implementation found");
			} catch (ClassCastException e) {
				throw new WebLayerIntegrationException("It should be an instance of ICrudDAO");
			}
		}else{
			logger.debug("No user pattern configured in "+ Constant.FRAMEWORK_CONFIGURATOR_FILE);
		}

		logger.debug("Injecting "+ImplicitSuperCrudDAO.class);
		return new ImplicitSuperCrudDAO(pojo,getEntityManager());
	}

	/**
	 * Retrieves the entity manager from PersistenceEntityManagerFactory
	 * @see br.gov.framework.demoiselle.persistence.layer.integration.PersistenceEntityManagerFactory
	 * @return
	 */
	private EntityManager getEntityManager() {
		InjectionContext ctx = new InjectionContext(annotation,EntityManager.class, ImplicitSuperCrudDAO.class);
		return new PersistenceEntityManagerFactory().create(ctx);

	}

	/**
	 * Just for simulate an injection.
	 */
	//TODO find a way to call the PersistenceEntityManagerFatory directly;
	private Injection annotation = new Injection() {
		public Class<? extends Annotation> annotationType() {
			return Injection.class;
		}
		public String[] parameters() {
			return new String[]{""};
		}
		public String name() {
			return "";
		}
		@SuppressWarnings("unchecked")
		public Class<? extends IFactory> factory() {
			return PersistenceEntityManagerFactory.class;
		}
	};
}
