/*
 * Demoiselle Framework
 * Copyright (c) 2010 Serpro and other contributors as indicated
 * by the @author tag. See the copyright.txt in the distribution for a
 * full listing of contributors.
 *
 * Demoiselle Framework is an open source Java EE library designed to accelerate
 * the development of transactional database Web applications.
 *
 * Demoiselle Framework is released under the terms of the LGPL license 3
 * http://www.gnu.org/licenses/lgpl.html  LGPL License 3
 *
 * This file is part of Demoiselle Framework.
 *
 * Demoiselle Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 3 as published by
 * the Free Software Foundation.
 *
 * Demoiselle Framework is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Demoiselle Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
package br.gov.component.demoiselle.crud.supercrud;

import java.io.Serializable;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.faces.context.FacesContext;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Transient;

import br.gov.component.demoiselle.crud.annotation.Input;
import br.gov.component.demoiselle.crud.annotation.Label;
import br.gov.component.demoiselle.crud.util.Strings;

/**
 * Visual representation of super models
 *
 * @author SERPRO/CETEC
 */
public final class Element implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * String representation of the label for a visual element
	 */
	private String label;

	/**
	 * ConverterId for a visual element
	 */
	private String converterId;

	/**
	 * Id for a visual element
	 */
	private String id;

	/**
	 * Integer representation of the length for a visual element
	 */
	private Integer length;

	/**
	 * Information source for a visual element
	 */
	@Transient
	private final Field field;

	/**
	 * Indicates whether the element is editable
	 */
	private Boolean showOnly;

	/**
	 * Corresponds to the visual JSF representation of the element
	 */
	private String input;
	private String inputTemplate;
	private List<InputAttribute> inputAttributes = new ArrayList<InputAttribute>();

	/**
	 * Creates a element from source field
	 *
	 * @param field
	 */
	public Element(final Field field) {
		this.field = field;
		this.inputTemplate = "";
		loadInputTypeInfo();
	}

	private void loadInputTypeInfo() {
		for (Annotation a : this.field.getAnnotations()) {
			if (a.annotationType().isAnnotationPresent(Input.class)) {
				this.input = a.annotationType().getSimpleName();
				Input inputAnnotation = a.annotationType().getAnnotation(Input.class);
				if (inputAnnotation != null) {
					this.inputTemplate = inputAnnotation.value();
				}
				for (Method aMethod : a.annotationType().getDeclaredMethods()) {
					try {
						String value = aMethod.invoke(a).toString();
						inputAttributes.add(new InputAttribute(aMethod
								.getName(), value));
					} catch (Exception e) {
						//TODO tratar exceção
						e.printStackTrace();
					}
				}
				break;
			}
		}

		if (this.input == null)
			this.input = "text";
		else
			// lowercases first letter (transform InputTypeName into inputTypeName)
			this.input = input.substring(0,1).toLowerCase() + input.substring(1);
	}

	/**
	 * @see #converterId
	 * @return converterId
	 */
	public String getConverterId() {

		if (this.converterId == null) {
			this.converterId = null;
		}

		return this.converterId;
	}

	/**
	 * Returns the id for a visual element
	 */
	public String getId() {
		if (this.id == null) {
			this.id = this.field.getName();
		}

		return this.id;
	}

	/**
	 * Returns the label for a visual element
	 */
	public String getLabel() {

		if (this.label == null) {
			if (this.field.isAnnotationPresent(Label.class)) {
				this.label = this.field.getAnnotation(Label.class).value();
				if (this.label.matches("^\\{.*\\}$")) {
					String key = this.label.substring(1,
							this.label.length() - 1);
					Locale locale = FacesContext.getCurrentInstance().getExternalContext().getRequestLocale();
					this.label = Strings.getFromResourceBundle(key, locale);
				}

			} else {
				this.label = Strings.capitalize(this.field.getName());
			}

		}

		return this.label;
	}

	/**
	 * Returns the field corresponding Input
	 *
	 * @return
	 */
	public String getInput() {
		return input;
	}

	/**
	 * Returns the field corresponding Input template xhtml file name
	 *
	 * @return
	 */
	public String getInputTemplate() {
		return inputTemplate;
	}

	/**
	 * Returns the Input Type Attribute value. Null, if attributeName was not
	 * found.
	 *
	 * @return
	 */
	public String getInputAttribute(String attributeName) {
		for (InputAttribute attr : inputAttributes)
			if (attr.getName().equals(attributeName))
				return attr.getValue();
		return null;
	}

	/**
	 * Returns the Input Type Attribute value
	 *
	 * @return
	 */
	public List<InputAttribute> getInputAttributes() {
		return inputAttributes;
	}

	/**
	 * Returns the length for a visual element
	 */
	public Integer getLength() {
		if (this.length == null) {
			this.length = null;
		}

		return this.length;
	}

	/**
	 * Returns the property that indicates whether the element is editable
	 */
	public Boolean getShowOnly() {
		if (this.showOnly == null) {
			this.showOnly = this.field
					.isAnnotationPresent(GeneratedValue.class)
					|| this.field.isAnnotationPresent(Id.class);
		}

		return this.showOnly;
	}
}
