/*
 * Demoiselle Framework
 * Copyright (c) 2010 Serpro and other contributors as indicated
 * by the @author tag. See the copyright.txt in the distribution for a
 * full listing of contributors.
 *
 * Demoiselle Framework is an open source Java EE library designed to accelerate
 * the development of transactional database Web applications.
 *
 * Demoiselle Framework is released under the terms of the LGPL license 3
 * http://www.gnu.org/licenses/lgpl.html  LGPL License 3
 *
 * This file is part of Demoiselle Framework.
 *
 * Demoiselle Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 3 as published by
 * the Free Software Foundation.
 *
 * Demoiselle Framework is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Demoiselle Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
package br.gov.component.demoiselle.crud.supercrud;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.List;

import javax.persistence.Id;

import br.gov.component.demoiselle.crud.annotation.NotFilter;
import br.gov.component.demoiselle.crud.exception.DemoiselleCrudException;
import br.gov.component.demoiselle.crud.layer.ICrudDAO;
import br.gov.component.demoiselle.crud.message.CrudMessages;
import br.gov.component.demoiselle.crud.util.GenericsReader;
import br.gov.framework.demoiselle.core.bean.IPojo;
import br.gov.framework.demoiselle.persistence.JPAGenericDAO;

/**
 * Default implementation for the {@link ICrudDAO} interface.
 *
 * @param <P> Pojo
 * @param <I> Pojo ID
 * @param <F> Filter
 *
 * @author SERPRO/CETEC
 */
public class SuperCrudDAO<P extends IPojo> extends JPAGenericDAO<P> implements ICrudDAO<P> {

	private Class<P> pojoClass;

	public Object insert(P pojo) {
		super.insert(pojo);
		super.flushAndClear();
		return pojo;
	}
	
	public void update(P pojo) {
		super.update(pojo);
		super.flushAndClear();
	}

	public void remove(P pojo) {
		P aux = null;

		boolean accessible;
		for (Field field : pojo.getClass().getDeclaredFields()) {
			if (field.isAnnotationPresent(Id.class)) {
				try {
					accessible = field.isAccessible();
					field.setAccessible(true);
					aux = this.findById((Serializable) field.get(pojo));
					field.setAccessible(accessible);

				} catch (IllegalArgumentException e) {
					e.printStackTrace();
					throw new DemoiselleCrudException(CrudMessages.CRUD_FIND_BY_FILTER_NOK);
				} catch (IllegalAccessException e) {
					e.printStackTrace();
					throw new DemoiselleCrudException(CrudMessages.CRUD_FIND_BY_FILTER_NOK);
				}

				break;
			}
		}

		if (aux != null) {
			super.remove(aux);
		}
	}

	/**
	 * Serialization Version UID
	 */
	private static final long serialVersionUID = 1L;

	public List<P> find(final P filter) {
		
		StringBuffer mainQL = new StringBuffer();
		StringBuffer restrictionQL = new StringBuffer();

		mainQL.append(getFindAllQuery(filter.getClass()));

		boolean accessible;
		Object value;
		
		if (filter != null) {
			for (Field field : getPojoClass().getDeclaredFields()) {
				if (!field.isAnnotationPresent(NotFilter.class) && !Modifier.isStatic(field.getModifiers())) {
					try {
						accessible = field.isAccessible();
						field.setAccessible(true);
						value = field.get(filter);
						field.setAccessible(accessible);

						if (value != null && !value.toString().trim().equals("")) {
							if (restrictionQL.length() > 0) {
								restrictionQL.append(" or ");
							}
							restrictionQL.append("p." + field.getName() + " like '%" + value + "%'");
						}

					} catch (Exception cause) {
						cause.printStackTrace();
						throw new DemoiselleCrudException(CrudMessages.CRUD_FIND_BY_FILTER_NOK);
					}
				}
			}
		}
		if (restrictionQL.length() > 0) {
			mainQL.append(" where ");
			mainQL.append(restrictionQL);
		}

		return this.findByJPQL(mainQL.toString());
	}

	/**
	 * The internal JPA query statement used for retrieve all Pojo from the persistent unit
	 *
	 * @return JPA query statement
	 */
	protected String getFindAllQuery(Class<?> pojo) {
		return "select p from " + pojo.getSimpleName() + " p";
	}

	/**
	 * @see #pojoClass
	 * @return Pojo Class
	 */
	protected Class<P> getPojoClass() {

		if (this.pojoClass == null) {
			this.pojoClass = GenericsReader.getTypeArgument(this.getClass(), 0);
		}

		return this.pojoClass;
	}

}
