/*
 * Demoiselle Framework
 * Copyright (c) 2010 Serpro and other contributors as indicated
 * by the @author tag. See the copyright.txt in the distribution for a
 * full listing of contributors.
 *
 * Demoiselle Framework is an open source Java EE library designed to accelerate
 * the development of transactional database Web applications.
 *
 * Demoiselle Framework is released under the terms of the LGPL license 3
 * http://www.gnu.org/licenses/lgpl.html  LGPL License 3
 *
 * This file is part of Demoiselle Framework.
 *
 * Demoiselle Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 3 as published by
 * the Free Software Foundation.
 *
 * Demoiselle Framework is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Demoiselle Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
package br.gov.component.demoiselle.crud.supercrud;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

import javax.faces.context.FacesContext;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;

import br.gov.component.demoiselle.crud.annotation.NotField;
import br.gov.component.demoiselle.crud.annotation.NotFilter;
import br.gov.component.demoiselle.crud.annotation.NotList;
import br.gov.component.demoiselle.crud.exception.DemoiselleCrudException;
import br.gov.component.demoiselle.crud.layer.ICrudBC;
import br.gov.component.demoiselle.crud.layer.ICrudMB;
import br.gov.component.demoiselle.crud.layer.integration.CrudBusinessControllerFactory;
import br.gov.component.demoiselle.crud.message.CrudMessages;
import br.gov.component.demoiselle.crud.message.MessageDecorator;
import br.gov.component.demoiselle.crud.template.AbstractPageMB;
import br.gov.component.demoiselle.crud.util.GenericsReader;
import br.gov.framework.demoiselle.core.bean.IPojo;
import br.gov.framework.demoiselle.core.layer.integration.Injection;
import br.gov.framework.demoiselle.view.faces.controller.ControllerException;
import br.gov.framework.demoiselle.view.faces.util.ManagedBeanUtil;

/**
 * No Abstract ICrudMB implementation, representing a Page that contains a Pojo with their properties
 *
 * @param <P> Pojo
 *
 * @author SERPRO/CETEC
 */
public class SuperCrudMB<P extends IPojo> extends AbstractPageMB implements ICrudMB<P> {

	/**
	 * Serialization Version UID
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Generic business controller
	 */
	@Injection(factory=CrudBusinessControllerFactory.class)
	private ICrudBC<P> bc;

	/**
	 * Controls filter On/Off
	 */
	private boolean filterOn = false;

	/**
	 * Fields collection that will be shown in crud page for edition.
	 */
	private List<Element> fields;

	/**
	 * Pojo ID
	 */
	private Serializable id;

	/**
	 * Pojo instance
	 */
	private P pojo;

	/**
	 * Generic Pojo classifier Class Type
	 */
	private Class<P> pojoClass;

	/**
	 * Generic filter
	 */
	private P filter;

	/**
	 * Filers collection that will be shown list page.
	 */
	private List<Element> filters;

	/**
	 * Wrapper to {@link #resultList}
	 */
	private DataModel dataModel;

	/**
	 * Pojo resultant collection of listing process
	 */
	private List<P> resultList;

	/**
	 * Fields collection that will be shown in pojo list.
	 */
	private List<Element> columns;

	/**
	 * Clear local variables.
	 */
	protected void clear() {
		this.dataModel = null;
		this.resultList = null;
		this.id = null;
		this.pojo = null;
	}

	/**
	 * Clear the generic filter
	 */
	public void clearFilter() {
		this.filter = newFilter();
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.ICrudMB#delete()
	 */
	public String delete() {
		String targetView = null;
		try {
			targetView = this.handleDelete();
			ManagedBeanUtil.addMessage(new MessageDecorator(CrudMessages.CRUD_DELETE_OK, getManagedBeanName()));
		} catch (DemoiselleCrudException e) {
			ManagedBeanUtil.addMessage(new MessageDecorator(CrudMessages.CRUD_DELETE_NOK, getManagedBeanName()),e);
			e.printStackTrace();
		}
		return targetView;
	}

	public boolean getGlobalMessages() {
		return FacesContext.getCurrentInstance().getMessages(null).hasNext();
	}
 
	/**
	 * Returns generic business controller
	 */
	protected ICrudBC<P> getBC() {
		return this.bc;
	}


	/**
	 * Extract all Fields from Pojo and convert them to Property. This is used for assembly the Table columns.
	 *
	 * @return
	 */
	public List<Element> getColumns() {
		if (this.columns == null) {
			this.columns = new ArrayList<Element>();
			for (Field field : getPojoClass().getDeclaredFields()) {
				if (!field.isAnnotationPresent(NotList.class) && !Modifier.isStatic(field.getModifiers())) {
					this.columns.add(new Element(field));
				}
			}
		}
		return this.columns;
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.IListMB#getDataModel()
	 */
	public DataModel getDataModel() {
		if (this.dataModel == null) {
			this.dataModel = new ListDataModel(getResultList());
		}
		return this.dataModel;
	}

	/**
	 * Extract all Fields from Pojo and convert them to Property. This is used for assembly the crud page.
	 *
	 * @return
	 */
	public List<Element> getFields() {
		if (this.fields == null) {
			this.fields = new ArrayList<Element>();

			for (java.lang.reflect.Field field : getPojoClass().getDeclaredFields()) {
				if (!field.isAnnotationPresent(NotField.class) && !Modifier.isStatic(field.getModifiers())) {
					this.fields.add(new Element(field));
				}
			}
		}

		return this.fields;
	}

	/**
	 * Return the generic filter
	 */
	public P getFilter() {
		if (this.filter == null) {
			this.filter = newFilter();
		}
		return this.filter;
	}

	/**
	 * Extract all Fields from Pojo and convert them to Property. This is used for assembly the filter form.
	 *
	 * @return
	 */
	public List<Element> getFilters() {
		if (this.filters == null) {
			this.filters = new ArrayList<Element>();

			for (Field field : getPojoClass().getDeclaredFields()) {
				if (!field.isAnnotationPresent(NotFilter.class) && !Modifier.isStatic(field.getModifiers())) {
					this.filters.add(new Element(field));
				}
			}
		}

		return this.filters;
	}

	/**
	 * Returns the Pojo ID.
	 */
	public Serializable getId() {
		return this.id;
	}

	/**
	 * Returns the Pojo current instance.
	 */
	public P getPojo() {
		if (this.pojo == null) {
			initPojo();
		}

		return this.pojo;
	}

	/**
	 * @see #pojoClass
	 * @return Pojo Class
	 */
	protected Class<P> getPojoClass() {
		if (this.pojoClass == null) {
			this.pojoClass = GenericsReader.getTypeArgument(this.getClass(), 0);
		}

		return this.pojoClass;
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.IListMB#getResultList()
	 */
	public List<P> getResultList() {
		if (this.resultList == null) {
			try {
				this.resultList = handleResultList();
			} catch (DemoiselleCrudException e) {
				ManagedBeanUtil.addMessage(new MessageDecorator(CrudMessages.CRUD_FIND_NOK, getManagedBeanName()),e);
				e.printStackTrace();
			}
		}

		return this.resultList;
	}

	public void setResultList(List<P> resultList) {
		this.resultList = resultList;
	}

	protected String handleDelete() throws DemoiselleCrudException {
		getBC().delete(getPojo());
		clear();
		return getPreviousView();
	}

	protected String handleInsert() throws DemoiselleCrudException {
		getBC().insert(getPojo());
		clear();
		return getPreviousView();
	}

	protected List<P> handleResultList() throws DemoiselleCrudException {
		return getBC().find(getFilter());
	}

	protected String handleUpdate() throws DemoiselleCrudException {
		getBC().update(getPojo());
		clear();
		return getPreviousView();
	}

	/**
	 * Inits the current Pojo instance.
	 */
	protected void initPojo() {
		if (isUpdateMode()) {
			try {
				this.pojo = this.loadPojo();
			} catch (DemoiselleCrudException e) {
				throw new ControllerException("Fail to initPojo");
			}
		} else {
			setPojo(newPojo());
		}
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.ICrudMB#insert()
	 */
	public String insert() {
		String targetView = null;
		try {
			targetView = this.handleInsert();
			ManagedBeanUtil.addMessage(new MessageDecorator(CrudMessages.CRUD_INSERT_OK, getManagedBeanName()));
		} catch (DemoiselleCrudException e) {
			ManagedBeanUtil.addMessage(new MessageDecorator(CrudMessages.CRUD_INSERT_NOK, getManagedBeanName()),e);
			e.printStackTrace();
		}
		return targetView;
	}

	@Override
	public boolean isCrudMB() {
		return true;
	}

	/**
	 * return true if Filter in On
	 */
	public boolean isFilterOn() {
		return this.filterOn;
	}

	public boolean isShowOnly() {
		return false;
	}

	/**
	 * Returns true if current Pojo is in update mode (user can update or delete).
	 *
	 * @return true or false
	 */
	public boolean isUpdateMode() {
		return getId() != null;
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.IListMB#list()
	 */
	public String list() {
		clear();
		return getCurrentView();
	}

	protected P loadPojo() throws DemoiselleCrudException {
		return getBC().load(getId());
	}

	/**
	 * Returns a new Instance of filter.
	 */
	private P newFilter() {
		try {
			return getPojoClass().newInstance();
		} catch (InstantiationException e) {
			throw new ControllerException("Could not instanciate "+getPojoClass(), e);
		} catch (IllegalAccessException e) {
			throw new ControllerException("IllegalAccessException in "+getPojoClass(), e);
		}
	}

	/**
	 * Creates a Pojo instance.
	 *
	 * @return Pojo
	 */
	private P newPojo() {
		P pojo = null;
		try {
			pojo = getPojoClass().newInstance();
		} catch (final InstantiationException e) {
			throw new ControllerException("Could not instanciate "+getPojoClass(), e);
		} catch (IllegalAccessException e) {
			throw new ControllerException("IllegalAccessException in "+getPojoClass(), e);
		}
		return pojo;
	}

	public String prepareDelete() {
		initPojo();
		return (getNavigationRules() == null) ? null : getEditCurrentView();
	}

	public String prepareInsert() {
		clear();
		return (getNavigationRules() == null) ? null : getEditCurrentView();
	}

	public String prepareUpdate() {
		initPojo();
		return (getNavigationRules() == null) ? null : getEditCurrentView();
	}

	/**
	 * Set filter On / Off
	 *
	 * @param filterOn
	 *            true - turns filter On
	 */
	public void setFilterOn(final boolean filterOn) {
		this.filterOn = filterOn;
	}

	/**
	 * Sets the Pojo ID.
	 *
	 * @param id
	 */
	public void setId(final Serializable id) {
		clear();
		this.id = id;
	}

	/**
	 * Sets the Pojo instance.
	 *
	 * @param pojo
	 */
	protected void setPojo(final P pojo) {
		this.pojo = pojo;
	}

	public String show() {
		return (getNavigationRules() == null) ? null : super.getShowCurrentView();
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.ICrudMB#update()
	 */
	public String update() {
		String targetView = null;
		try {
			targetView = this.handleUpdate();
			ManagedBeanUtil.addMessage(new MessageDecorator(CrudMessages.CRUD_UPDATE_OK, getManagedBeanName()));
		} catch (DemoiselleCrudException e) {
			ManagedBeanUtil.addMessage(new MessageDecorator(CrudMessages.CRUD_UPDATE_NOK, getManagedBeanName()),e);
			e.printStackTrace();
		}
		return targetView;
	}

}
