/*
 * Demoiselle Framework Copyright (c) 2009 Serpro and other contributors as indicated by the @author tag. See the
 * copyright.txt in the distribution for a full listing of contributors. Demoiselle Framework is an open source Java EE
 * library designed to accelerate the development of transactional database Web applications. Demoiselle Framework is
 * released under the terms of the LGPL license 3 http://www.gnu.org/licenses/lgpl.html LGPL License 3 This file is part
 * of Demoiselle Framework. Demoiselle Framework is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License 3 as published by the Free Software Foundation. Demoiselle Framework
 * is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details. You
 * should have received a copy of the GNU Lesser General Public License along with Demoiselle Framework. If not, see
 * <http://www.gnu.org/licenses/>.
 */
package br.gov.component.demoiselle.crud.template;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;

import javax.faces.context.FacesContext;

import org.apache.log4j.Logger;

import br.gov.component.demoiselle.crud.annotation.NavigationRule;
import br.gov.component.demoiselle.crud.annotation.Page;
import br.gov.component.demoiselle.crud.annotation.PageType;
import br.gov.component.demoiselle.crud.layer.IPageMB;
import br.gov.component.demoiselle.crud.util.Strings;
import br.gov.framework.demoiselle.view.faces.controller.AbstractManagedBean;
import br.gov.framework.demoiselle.view.faces.controller.AbstractManagedBeanConfig;

/**
 * Abstract IPageMB implementation, that represents a generic Web Page.
 *
 * @author SERPRO/CETEC
 */
public abstract class AbstractPageMB extends AbstractManagedBean implements IPageMB, Serializable {

	/**
	 * Serialization Version UID
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Provide the default logger for all pages
	 */
	private static Logger log = Logger.getLogger(AbstractManagedBean.class);

	/**
	 * Contains all ManagedBean configurations values
	 */
	private static AbstractManagedBeanConfig config = AbstractManagedBeanConfig.getInstance();

	protected Page page;

	private String listPreviousView;

	private String editPreviousView;

	private String showPreviousView;

	private String listCurrentView;

	private String editCurrentView;

	private String showCurrentView;

	private String listTitle;

	private String editTitle;

	private String showTitle;

	/**
	 * NavigationRules
	 */
	private List<NavigationRule> navigationRuleList;

	public AbstractPageMB() {
		this.page = this.getClass().getAnnotation(Page.class);

		if (this.page != null) {
			if (this.page.navigationRules() != null) {
				this.navigationRuleList = Arrays.asList(this.page.navigationRules());
				for (NavigationRule navigationRule : getNavigationRules()) {
					if (navigationRule.type() == PageType.LIST) {
						this.listTitle = navigationRule.title();
						this.listCurrentView = navigationRule.currentView();
						this.listPreviousView = !navigationRule.previousView().equals("") ? navigationRule
								.previousView() : this.listCurrentView;
					} else if (navigationRule.type() == PageType.EDIT) {
						this.editTitle = navigationRule.title();
						this.editCurrentView = navigationRule.currentView();
						this.editPreviousView = !navigationRule.previousView().equals("") ? navigationRule
								.previousView() : this.editCurrentView;
					} else if (navigationRule.type() == PageType.SHOW) {
						this.showTitle = navigationRule.title();
						this.showCurrentView = navigationRule.currentView();
						this.showPreviousView = !navigationRule.previousView().equals("") ? navigationRule
								.previousView() : this.showCurrentView;
					}
				}

				// Page <- List <- Edit <- Show
				if (this.listTitle == null)
					this.listTitle = this.page.title();
				if (this.listCurrentView == null)
					this.listCurrentView = this.page.currentView();
				if (this.listPreviousView == null)
					this.listPreviousView = this.page.previousView();

				if (this.editTitle == null)
					this.editTitle = this.listTitle;
				if (this.editCurrentView == null)
					this.editCurrentView = this.listCurrentView;
				if (this.editPreviousView == null)
					this.showPreviousView = this.listPreviousView;

				if (this.showTitle == null)
					this.showTitle = this.editTitle;
				if (this.showCurrentView == null)
					this.showCurrentView = this.editCurrentView;
				if (this.showPreviousView == null)
					this.showPreviousView = this.editPreviousView;
			} else {
				this.listTitle = this.page.title();
				this.editTitle = this.page.title();
				this.showTitle = this.page.title();
				this.listCurrentView = this.page.currentView();
				this.editCurrentView = this.page.currentView();
				this.showCurrentView = this.page.currentView();
				this.listPreviousView = this.page.previousView();
				this.editPreviousView = this.page.previousView();
				this.showPreviousView = this.page.previousView();
			}
		}
	}

	/**
	 * @see #config
	 * @return AbstractManagedBeanConfig
	 */
	protected AbstractManagedBeanConfig getConfig() {
		return config;
	}

	/**
	 * Returns the maximum number of pages.
	 *
	 * @return number of pages
	 */
	@Override
	public Integer getMaxPages() {
		return getConfig().getMaxPages();
	}

	/**
	 * Returns the number of records per page.
	 *
	 * @return number of records
	 */
	public Integer getMaxRows() {
		return getConfig().getRows();
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.IPageMB#getCurrentView()
	 */
	public String getCurrentView() {
		if (this.page != null) {
			return this.page.currentView();
		}
		return null;
	}

	public String getEditCurrentView() {
		return this.editCurrentView;
	}

	public String getEditPreviousView() {
		return this.editPreviousView;
	}

	public String getEditTitle() {
		return getTitleResourceBundle(this.editTitle);
	}

	public String getListCurrentView() {
		return this.listCurrentView;
	}

	public String getListPreviousView() {
		return this.listPreviousView;
	}

	public String getListTitle() {
		return getTitleResourceBundle(this.listTitle);
	}

	/**
	 * @see #log
	 * @return Logger
	 */
	protected Logger getLog() {
		return log;
	}

	/**
	 * Returns the simple name of this managed bean whith first letter in lower case;
	 *
	 * @return
	 */
	protected String getManagedBeanName() {
		String beanName = getClass().getSimpleName();
		return beanName.substring(0, 1).toLowerCase() + beanName.substring(1);
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.IPageMB#getNavigationRules()
	 */
	public List<NavigationRule> getNavigationRules() {
		return this.navigationRuleList;
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.IPageMB#getPreviousView()
	 */
	public String getPreviousView() {
		if (this.page != null) {
			return !this.page.previousView().equals("") ? this.page.previousView() : this.page.currentView();
		}
		return null;
	}

	public String getShowCurrentView() {
		return this.showCurrentView;
	}

	public String getShowPreviousView() {
		return this.showPreviousView;
	}

	public String getShowTitle() {
		return getTitleResourceBundle(this.showTitle);
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.IPageMB#getTitle()
	 */
	public String getTitle() {
		if (this.page != null) {
			return getTitleResourceBundle(this.page.title());
		}
		return null;
	}

	public String getTitleResourceBundle(final String keyOrValue) {
		if (keyOrValue.matches("^\\{.*\\}$")) {
			String key = keyOrValue.substring(1, keyOrValue.length() - 1);

			return Strings.getFromResourceBundle(key, FacesContext.getCurrentInstance().getViewRoot().getLocale());
		}
		return keyOrValue;
	}

	/*
	 * (non-Javadoc)
	 * @see br.gov.component.demoiselle.crud.layer.IPageMB#isCrudMB()
	 */
	public boolean isCrudMB() {
		return false;
	}

}
